/*
 * Tapioca library
 * Copyright (C) 2005 INdT 
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <tapioca/base/tapioca_marshal.h>

#include "tapioca_privacy.h"
#include "tapioca_privacy_bindings.h"

#define INTERFACE_NAME "org.tapioca.Privacy"

/* Signals */
enum {
    RULE_RETRIEVED,
    LISTS_RETRIEVED,
    LAST_SIGNAL
};

/* Private members */
struct _TapiocaPrivacyPrivate {
    gboolean disposed;
    DBusGProxy *proxy;
};

static void tapioca_privacy_dispose        (GObject *object);
static void tapioca_privacy_finalize       (GObject *object);

static void proxy_rule_retrieved_signal    (DBusGProxy *proxy, 
                                            const gchar *list_name,
                                            guint type,
                                            const gchar *value,
                                            guint action,
                                            gpointer data);
static void proxy_lists_retrieved_signal   (DBusGProxy *proxy,  
                                            gpointer data);

G_DEFINE_TYPE(TapiocaPrivacy, tapioca_privacy, G_TYPE_OBJECT)
   
static guint tapioca_privacy_signals[LAST_SIGNAL] = { 0 };

static void 
tapioca_privacy_class_init (TapiocaPrivacyClass *klass) 
{
    GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;
    tapioca_privacy_parent_class = g_type_class_peek_parent (klass);
    
    g_type_class_add_private (klass, sizeof (TapiocaPrivacyPrivate));
    
    gobject_class->dispose = tapioca_privacy_dispose;
    gobject_class->finalize = tapioca_privacy_finalize;
    
    /* Let's create our signals */
    tapioca_privacy_signals[RULE_RETRIEVED] = g_signal_new ("rule-retrieved",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
        0,
        NULL,
        NULL,
        tapioca_marshal_VOID__STRING_UINT_STRING_UINT,
        G_TYPE_NONE,
        4,
        G_TYPE_STRING,
        G_TYPE_UINT,
        G_TYPE_STRING,
        G_TYPE_UINT);
    tapioca_privacy_signals[LISTS_RETRIEVED] = g_signal_new ("lists-retrieved",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
        0,
        NULL,
        NULL,
        g_cclosure_marshal_VOID__POINTER,
        G_TYPE_NONE,
        1,
        G_TYPE_STRV);
}

static void 
tapioca_privacy_init (TapiocaPrivacy *tp) 
{
    tp->priv = TAPIOCA_PRIVACY_GET_PRIVATE (tp);
    tp->priv->proxy = NULL;
    tp->priv->disposed = FALSE;
}

static void 
tapioca_privacy_dispose (GObject *object) 
{
    TapiocaPrivacy *tp = TAPIOCA_PRIVACY (object);

    if (tp->priv->disposed) {
        /* If dispose did already run, return. */
        return;
    }
   
    /* Make sure dispose does not run twice. */
    tp->priv->disposed = TRUE;

    /* Disconnect all signals */
    if (tp->priv->proxy) {
        dbus_g_proxy_disconnect_signal (tp->priv->proxy, "RuleRetrieved",
                G_CALLBACK (proxy_rule_retrieved_signal), NULL);
        dbus_g_proxy_disconnect_signal (tp->priv->proxy, "ListsRetrieved",
                G_CALLBACK (proxy_lists_retrieved_signal), NULL);

        g_object_unref (tp->priv->proxy);
    }

    G_OBJECT_CLASS (tapioca_privacy_parent_class)->dispose (object);
}

static void 
tapioca_privacy_finalize (GObject *object) 
{
    g_signal_handlers_destroy (object);
    
    G_OBJECT_CLASS (tapioca_privacy_parent_class)->finalize (object);
}

TapiocaPrivacy * 
tapioca_privacy_new (DBusGConnection *bus,
                      const gchar *service_name, 
                      const gchar *obj_path)
{
    TapiocaPrivacy *tp;
    DBusGProxy *proxy;

    /* Request a new proxy for tp service */
    proxy = dbus_g_proxy_new_for_name (bus,
            service_name,
            obj_path,
            INTERFACE_NAME);
    if (!proxy) {
        g_warning ("failed to create proxy for service %s with object path %s on interface %s", 
                service_name, obj_path, INTERFACE_NAME);
        return NULL;
    }

    tp = TAPIOCA_PRIVACY (g_object_new (tapioca_privacy_get_type (), NULL));
    tp->priv->proxy = proxy;

    /* Connect all tp signals */
    dbus_g_proxy_add_signal (tp->priv->proxy, "RuleRetrieved", G_TYPE_STRING, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_INVALID);
    dbus_g_proxy_add_signal (tp->priv->proxy, "ListsRetrieved", G_TYPE_STRV, G_TYPE_INVALID);

    dbus_g_proxy_connect_signal (tp->priv->proxy, "RuleRetrieved",
            G_CALLBACK (proxy_rule_retrieved_signal),
            tp, NULL);
    dbus_g_proxy_connect_signal (tp->priv->proxy, "ListsRetrieved",
            G_CALLBACK (proxy_lists_retrieved_signal),
            tp, NULL);

    return tp;
}

/**
 * Request privacy lists available.
 *
 * @param tp #TapiocaPrivacy instance.
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_request_lists (TapiocaPrivacy *tp)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);

    if (!(org_tapioca_Privacy_request_lists (tp->priv->proxy, &error))) {
        g_warning ("failed to request privacy lists");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

/**
 * Request privacy rules of a given list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @param name List name.
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_request_list_rules (TapiocaPrivacy *tp,
                                    const gchar *name)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);
    g_return_val_if_fail (name != NULL, FALSE);

    if (!(org_tapioca_Privacy_request_list_rules (tp->priv->proxy, name, &error))) {
        g_warning ("failed to request list rules");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

/**
 * Request default privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @return The name of default list.
 */
gchar
*tapioca_privacy_get_default_list (TapiocaPrivacy *tp)
{
    GError *error = NULL;
    gchar *name;

    g_return_val_if_fail (tp != NULL, NULL);

    if (!(org_tapioca_Privacy_get_default_list (tp->priv->proxy, &name, &error))) {
        g_warning ("failed to get default list");
        return NULL;
    }

    if (error)
        g_error_free (error);

    return name;
}

/**
 * Set default privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @param name List name.
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_set_default_list (TapiocaPrivacy *tp,
                                  const gchar *name)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);
    g_return_val_if_fail (name != NULL, FALSE);

    if (!(org_tapioca_Privacy_set_default_list (tp->priv->proxy, name, &error))) {
        g_warning ("failed to set default list");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

/**
 * Request active privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @return The name of active list.
 */
gchar
*tapioca_privacy_get_active_list (TapiocaPrivacy *tp)
{
    GError *error = NULL;
    gchar *name;

    g_return_val_if_fail (tp != NULL, NULL);

    if (!(org_tapioca_Privacy_get_active_list (tp->priv->proxy, &name, &error))) {
        g_warning ("failed to get active list");
        return NULL;
    }

    if (error)
        g_error_free (error);

    return name;
}

/**
 * Set active privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @param name List name.
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_set_active_list (TapiocaPrivacy *tp,
                                 const gchar *name)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);
    g_return_val_if_fail (name != NULL, FALSE);

    if (!(org_tapioca_Privacy_set_active_list (tp->priv->proxy, name, &error))) {
        g_warning ("failed to set active list");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

/**
 * Add a rule to a privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @param list_name List name.
 * @param rule Rule type
 * @param value Rule value
 * @param action Rule action
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_add_list_rule (TapiocaPrivacy *tp,
                               const gchar *list_name,
                               guint type,
                               const gchar *value,
                               guint action)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);
    g_return_val_if_fail (list_name != NULL, FALSE);
    g_return_val_if_fail (value != NULL, FALSE);

    if (!(org_tapioca_Privacy_add_list_rule (tp->priv->proxy, list_name, type, value, action, &error))) {
        g_warning ("failed to add rule");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

/**
 * Remove a rule of a privacy list.
 *
 * @param tp #TapiocaPrivacy instance.
 * @param list_name List name.
 * @param rule Rule type
 * @param value Rule value
 * @param action Rule action
 * @return FALSE if an error occurs otherwise TRUE.
 */
gboolean
tapioca_privacy_remove_list_rule (TapiocaPrivacy *tp,
                                  const gchar *list_name,
                                  guint type,
                                  const gchar *value,
                                  guint action)
{
    GError *error = NULL;

    g_return_val_if_fail (tp != NULL, FALSE);
    g_return_val_if_fail (list_name != NULL, FALSE);
    g_return_val_if_fail (value != NULL, FALSE);

    if (!(org_tapioca_Privacy_remove_list_rule (tp->priv->proxy, list_name, type, value, action, &error))) {
        g_warning ("failed to remove rule");
        return FALSE;
    }

    if (error)
        g_error_free (error);

    return TRUE;
}

static void
proxy_rule_retrieved_signal (DBusGProxy *proxy, 
                             const gchar *list_name,
                             guint type,
                             const gchar *value,
                             guint action,
                             gpointer data)
{
    g_signal_emit (TAPIOCA_PRIVACY (data), tapioca_privacy_signals[RULE_RETRIEVED], 0, 
        list_name, type, value, action);
}

static void
proxy_lists_retrieved_signal   (DBusGProxy *proxy,  
                                gpointer data)
{
    /* TODO */
}
