/*
 * Tapioca library
 * Copyright (C) 2005 INdT 
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "tapioca_contact_list.h"
#include "tapioca_contact_list_private.h"
#include "tapioca_contact_list_glue.h"

#include <tapioca/base/tapioca_marshal.h>
#include <tapioca/base/tapioca_errors.h>
#include <tapioca/base/tapioca_enums.h>
#include <tapioca/base/tapioca_utils.h>

#include <dbus/dbus-glib-bindings.h>

enum {
    AUTHORIZATION_REQUESTED,
    CONTACT_AUTHORIZED,
    CONTACT_DENIED,
    CONTACTS_RETRIEVED,
    CONTACT_INFO_RETRIEVED,
    LAST_SIGNAL
};

struct _TapiocaContactListPrivate 
{
    GMutex *mutex;
    GHashTable *contact_list;
    gboolean subscribed;
    gboolean disposed;
};

G_DEFINE_TYPE(TapiocaContactList, tapioca_contact_list, G_TYPE_OBJECT)
    
static void tapioca_contact_list_dispose  (GObject *object);
static void tapioca_contact_list_finalize (GObject *object);

static guint tapioca_contact_list_signals[LAST_SIGNAL] = { 0 };

static void
tapioca_contact_list_class_init (TapiocaContactListClass *klass)
{
    GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;
    tapioca_contact_list_parent_class = g_type_class_peek_parent (klass);

    g_type_class_add_private (klass, sizeof (TapiocaContactListPrivate));
   
    gobject_class->dispose = tapioca_contact_list_dispose;
    gobject_class->finalize = tapioca_contact_list_finalize;
    
    klass->request_contacts = NULL;
    klass->request_contact_info = NULL;
    klass->add_contact = NULL;
    klass->edit_contact = NULL;
    klass->remove_contact = NULL;  
    klass->authorize_contact = NULL;
    klass->deny_contact = NULL;
            
    tapioca_contact_list_signals[AUTHORIZATION_REQUESTED] = g_signal_new ("authorization-requested",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            g_cclosure_marshal_VOID__STRING,
            G_TYPE_NONE,
            1,
            G_TYPE_STRING);
    tapioca_contact_list_signals[CONTACT_AUTHORIZED] = g_signal_new ("contact-authorized",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            g_cclosure_marshal_VOID__STRING,
            G_TYPE_NONE,
            1,
            G_TYPE_STRING);
    tapioca_contact_list_signals[CONTACT_DENIED] = g_signal_new ("contact-denied",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            g_cclosure_marshal_VOID__STRING,
            G_TYPE_NONE,
            1,
            G_TYPE_STRING);
    tapioca_contact_list_signals[CONTACTS_RETRIEVED] = g_signal_new ("contacts-retrieved",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            tapioca_marshal_VOID__STRING_STRING_STRING_UINT,
            G_TYPE_NONE,
            4,
            G_TYPE_STRING,
            G_TYPE_STRING,
            G_TYPE_STRING,
            G_TYPE_UINT);
    tapioca_contact_list_signals[CONTACT_INFO_RETRIEVED] = g_signal_new ("contact-info-retrieved",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            tapioca_marshal_VOID__STRING_STRING,
            G_TYPE_NONE,
            2,
            G_TYPE_STRING,
            G_TYPE_STRING);

    dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (klass), 
            &dbus_glib_tapioca_contact_list_object_info);
}

static void
tapioca_contact_list_init (TapiocaContactList *tcl)
{
    tcl->priv = TAPIOCA_CONTACT_LIST_GET_PRIVATE (tcl);
    tcl->priv->mutex = g_mutex_new ();
    tcl->priv->contact_list = g_hash_table_new_full (g_str_hash, g_str_equal, 
            g_free, NULL);
}

static void
tapioca_contact_list_dispose (GObject *object)
{    
    TapiocaContactList *tcl = TAPIOCA_CONTACT_LIST (object);

    if (tcl->priv->disposed)
        return;

    tcl->priv->disposed = TRUE;
    
    g_mutex_free (tcl->priv->mutex);
    g_hash_table_destroy (tcl->priv->contact_list);

    G_OBJECT_CLASS (tapioca_contact_list_parent_class)->dispose (object);
}

static void
tapioca_contact_list_finalize (GObject *object)
{
    g_signal_handlers_destroy (object);

    G_OBJECT_CLASS (tapioca_contact_list_parent_class)->finalize (object);
}

gboolean
tapioca_contact_list_request_contacts (TapiocaContactList *tcl,
                                       GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    
    g_set_error_val_if_fail (klass->request_contacts != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->request_contacts (tcl);
    
    return TRUE;
}

gboolean
tapioca_contact_list_request_contact_info (TapiocaContactList *tcl,
                                           const gchar *address,
                                           GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    
    g_set_error_val_if_fail (klass->request_contact_info != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->request_contact_info (tcl, address);

    return TRUE;
}

gboolean
tapioca_contact_list_add_contact (TapiocaContactList *tcl,
                                  const gchar *address,
                                  const gchar *alias,
                                  const gchar **groups,
                                  GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    gpointer key = NULL, value = NULL;
    
    g_set_error_val_if_fail (klass->add_contact != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    g_mutex_lock (tcl->priv->mutex);
    if (g_hash_table_lookup_extended (tcl->priv->contact_list, address,
            &key, &value)) {
        if (value != TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_NONE) {
            g_mutex_unlock (tcl->priv->mutex);
            return TRUE;
        }
    }
    g_mutex_unlock (tcl->priv->mutex);
    
    /* Add new contact to ContactList or re-invite for subscription */
    klass->add_contact (tcl, address, alias, groups);

    return TRUE;
}

gboolean
tapioca_contact_list_edit_contact (TapiocaContactList *tcl,
                                   const gchar *address,
                                   const gchar *alias,
                                   const gchar **groups,
                                   GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    gpointer key = NULL, value = NULL;
    
    g_set_error_val_if_fail (klass->add_contact != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    g_mutex_lock (tcl->priv->mutex);
    if (!g_hash_table_lookup_extended (tcl->priv->contact_list, address,
            &key, &value)) {
        g_mutex_unlock (tcl->priv->mutex);
        return TRUE;
    }
    if (value == TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_NONE) {
        g_mutex_unlock (tcl->priv->mutex);
        return TRUE;
    }
    g_mutex_unlock (tcl->priv->mutex);
    
    /* Edit contact on ContactList */
    klass->edit_contact (tcl, address, alias, groups);

    return TRUE;
}

gboolean
tapioca_contact_list_remove_contact (TapiocaContactList *tcl,
                                     const gchar *address,
                                     GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    gpointer key = NULL, value = NULL;
    
    g_set_error_val_if_fail (klass->remove_contact != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    g_mutex_lock (tcl->priv->mutex);
    if (g_hash_table_lookup_extended (tcl->priv->contact_list, address,
                &key, &value)) {
        g_hash_table_remove (tcl->priv->contact_list, address);
    }
    g_mutex_unlock (tcl->priv->mutex);
        
    klass->remove_contact (tcl, address);

    return TRUE;
}

gboolean
tapioca_contact_list_deny_contact (TapiocaContactList *tcl,
                                   const gchar *address,
                                   GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    
    g_set_error_val_if_fail (klass->deny_contact != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->deny_contact (tcl, address);

    return TRUE;
}

gboolean
tapioca_contact_list_authorize_contact (TapiocaContactList *tcl,
                                        const gchar *address,
                                        const gchar **groups,
                                        GError **error)
{
    TapiocaContactListClass *klass = TAPIOCA_CONTACT_LIST_GET_CLASS (tcl);
    
    g_set_error_val_if_fail (klass->authorize_contact != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->authorize_contact (tcl, address, groups);

    return TRUE;
}

void     
tapioca_contact_list_signal_contact_authorized (TapiocaContactList *tcl,
                                                const gchar *address)
{
    g_return_if_fail (tcl != NULL);

    g_signal_emit (tcl, tapioca_contact_list_signals[CONTACT_AUTHORIZED], 0, address);
}

void     
tapioca_contact_list_signal_contact_denied (TapiocaContactList *tcl,
                                            const gchar *address)
{
    g_return_if_fail (tcl != NULL);

    g_signal_emit (tcl, tapioca_contact_list_signals[CONTACT_DENIED], 0, address);
}

void     
tapioca_contact_list_signal_authorization_requested (TapiocaContactList *tcl,
                                                     const gchar *address)
{
    g_return_if_fail (tcl != NULL);

    g_signal_emit (tcl, tapioca_contact_list_signals[AUTHORIZATION_REQUESTED], 0, address);
}

void     
tapioca_contact_list_signal_contacts_retrieved (TapiocaContactList *tcl,
                                                const gchar *address,
                                                const gchar *alias,
                                                const gchar *group,
                                                guint subscription_type)
{
    gpointer key = NULL, value = NULL;

    g_return_if_fail (tcl != NULL);

    g_mutex_lock (tcl->priv->mutex);
    if (!g_hash_table_lookup_extended (tcl->priv->contact_list, address,
                &key, &value)) {
        g_hash_table_insert (tcl->priv->contact_list, g_strdup (address), (gpointer) subscription_type);
    }
    g_mutex_unlock (tcl->priv->mutex);

    g_signal_emit (tcl, tapioca_contact_list_signals[CONTACTS_RETRIEVED], 
            0, address, alias, group, subscription_type);
}

void     
tapioca_contact_list_signal_contact_info_retrieved (TapiocaContactList *tcl,
                                                    const gchar *address,
                                                    const gchar *info)
{
    g_return_if_fail (tcl != NULL);

    g_signal_emit (tcl, tapioca_contact_list_signals[CONTACT_INFO_RETRIEVED], 
            0, address, info);
}
