/*
 * Tapioca library
 * Copyright (C) 2005 INdT 
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "tapioca_privacy.h"
#include "tapioca_privacy_private.h"
#include "tapioca_privacy_glue.h"

#include <tapioca/base/tapioca_marshal.h>
#include <tapioca/base/tapioca_errors.h>
#include <tapioca/base/tapioca_enums.h>
#include <tapioca/base/tapioca_utils.h>

#include <dbus/dbus-glib-bindings.h>

enum {
    RULE_RETRIEVED,
    LISTS_RETRIEVED,
    LAST_SIGNAL
};

struct _TapiocaPrivacyPrivate 
{
    GMutex *mutex;
    GHashTable *active_list;
    gboolean disposed;
};

G_DEFINE_TYPE(TapiocaPrivacy, tapioca_privacy, G_TYPE_OBJECT)
    
static void  tapioca_privacy_dispose  (GObject *object);
static void  tapioca_privacy_finalize (GObject *object);

static guint tapioca_privacy_signals[LAST_SIGNAL] = { 0 };

static void
tapioca_privacy_class_init (TapiocaPrivacyClass *klass)
{
    GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;
    tapioca_privacy_parent_class = g_type_class_peek_parent (klass);

    g_type_class_add_private (klass, sizeof (TapiocaPrivacyPrivate));
   
    gobject_class->dispose = tapioca_privacy_dispose;
    gobject_class->finalize = tapioca_privacy_finalize;
    
    klass->request_lists = NULL;
    klass->request_list_rules = NULL;
    klass->get_active_list = NULL;
    klass->set_active_list = NULL;
    klass->get_default_list = NULL;
    klass->set_default_list = NULL;
    klass->add_list_rule = NULL;
    klass->remove_list_rule = NULL;
    
    tapioca_privacy_signals[RULE_RETRIEVED] = g_signal_new ("rule-retrieved",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            tapioca_marshal_VOID__STRING_UINT_STRING_UINT,
            G_TYPE_NONE,
            4,
            G_TYPE_STRING,
            G_TYPE_UINT,
            G_TYPE_STRING,
            G_TYPE_UINT);
    tapioca_privacy_signals[LISTS_RETRIEVED] = g_signal_new ("lists-retrieved",
            G_TYPE_FROM_CLASS (klass),
            G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
            0,
            NULL,
            NULL,
            g_cclosure_marshal_VOID__POINTER,
            G_TYPE_NONE,
            1,
            G_TYPE_STRV);

            
    dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (klass), 
            &dbus_glib_tapioca_privacy_object_info);
}

static void
tapioca_privacy_init (TapiocaPrivacy *tp)
{
    tp->priv = TAPIOCA_PRIVACY_GET_PRIVATE (tp);
    tp->priv->mutex = g_mutex_new ();
    tp->priv->active_list = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_free);
}

static void
tapioca_privacy_dispose (GObject *object)
{    
    TapiocaPrivacy *tp = TAPIOCA_PRIVACY (object);

    if (tp->priv->disposed)
        return;

    tp->priv->disposed = TRUE;
    
    g_mutex_free (tp->priv->mutex);
    g_hash_table_destroy (tp->priv->active_list);

    G_OBJECT_CLASS (tapioca_privacy_parent_class)->dispose (object);
}

static void
tapioca_privacy_finalize (GObject *object)
{
    g_signal_handlers_destroy (object);

    G_OBJECT_CLASS (tapioca_privacy_parent_class)->finalize (object);
}

gboolean
tapioca_privacy_request_lists (TapiocaPrivacy *tp,
                               GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->request_lists != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->request_lists (tp);
    
    return TRUE;
}

gboolean
tapioca_privacy_request_list_rules (TapiocaPrivacy *tp,
                                    const gchar *list_name,
                                    GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->request_list_rules != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->request_list_rules (tp, list_name);
    
    return TRUE;
}

gboolean
tapioca_privacy_set_default_list (TapiocaPrivacy *tp,
                                  const gchar *list_name,
                                  GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->set_default_list != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->set_default_list (tp, list_name);

    return TRUE;
}

gboolean
tapioca_privacy_get_default_list (TapiocaPrivacy *tp,
                                  const gchar **list_name,
                                  GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->get_default_list != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->get_default_list (tp, list_name);

    return TRUE;
}

gboolean
tapioca_privacy_set_active_list (TapiocaPrivacy *tp,
                                 const gchar *list_name,
                                 GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->set_active_list != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->set_active_list (tp, list_name);

    return TRUE;
}

gboolean
tapioca_privacy_get_active_list (TapiocaPrivacy *tp,
                                 const gchar **list_name,
                                 GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->get_active_list != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->get_active_list (tp, list_name);

    return TRUE;
}

gboolean
tapioca_privacy_add_list_rule (TapiocaPrivacy *tp,
                               const gchar *list_name,
                               guint type,
                               const gchar *value,
                               guint action,
                               GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->add_list_rule != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->add_list_rule (tp, list_name, type, value, action);

    return TRUE;
}

gboolean
tapioca_privacy_remove_list_rule (TapiocaPrivacy *tp,
                                  const gchar *list_name,
                                  guint type,
                                  const gchar *value,
                                  guint action,
                                  GError **error)
{
    TapiocaPrivacyClass *klass = TAPIOCA_PRIVACY_GET_CLASS (tp);
    
    g_set_error_val_if_fail (klass->remove_list_rule != NULL, FALSE, error, TAPIOCA_ERROR, TAPIOCA_ERROR_NOT_IMPLEMENTED, "Function not implemented");

    klass->remove_list_rule (tp, list_name, type, value, action);

    return TRUE;
}

void
tapioca_privacy_signal_rule_retrieved (TapiocaPrivacy *tp,
                                       const gchar *list_name,
                                       guint type,
                                       const gchar *value,
                                       guint action)
{
    g_return_if_fail (tp != NULL);

    g_debug ("retrieving rule of list %s type=%d value=%s action=%d...", list_name, 
        type, value, action);
    g_signal_emit (tp, tapioca_privacy_signals[RULE_RETRIEVED], 0, list_name, type,
        value, action);
}

void
tapioca_privacy_signal_lists_retrieved (TapiocaPrivacy *tp,
                                        gchar **lists)
{
    g_return_if_fail (tp != NULL);

    g_debug ("retrieving privacy lists names %p...", lists);
    g_signal_emit (tp, tapioca_privacy_signals[LISTS_RETRIEVED], 0, lists);
}
